﻿using System;
using System.Collections.Generic;
using System.Text;

namespace TeamSystemPro.Samples.WebAccessLinkCreator
{
    public class LinkCreator
    {
        private Uri _baseAddress;

        public LinkCreator(Uri baseAddress)
        {
            // Example: http://tfs:8090

            _baseAddress = baseAddress;
        }

        public Uri OpenTeamProject(string teamProject)
        {
            // Excample: http://tfs:8090/index.aspx?pname=TeamProject

            return new Uri(String.Format("{0}index.aspx?pname={1}",
                _baseAddress.AbsoluteUri, teamProject));
        }

        public Uri OpenWorkItem(int workItemID)
        {
            // Example: http://tfs:8090/wi.aspx?id=123 

            return new Uri(String.Format("{0}wi.aspx?id={1}",
                _baseAddress.AbsoluteUri, workItemID));
        }

        public Uri CreateWorkItem(string teamProject, string workItemType)
        {
            // Example: http://tfs:8090/wi.aspx?pname=TeamProject&wit=Bug

            return new Uri(String.Format("{0}wi.aspx?pname={1}&wit={2}",
                _baseAddress.AbsoluteUri, teamProject, workItemType));
        }

        public Uri NewWorkItemQuery(string teamProject)
        {
            // Example: http://tfs:8090/qe.aspx?pname=TeamProject

            return new Uri(String.Format("{0}qe.aspx?pname={1}",
                _baseAddress.AbsoluteUri, teamProject));
        }

        public Uri EditWorkItemQuery(string teamProjects, string queryName, string scope)
        {
            // Example: http://tfs:8090/qe.aspx?pname=TeamProject&name=Team%30Bugs&scope=public

            return new Uri(
                String.Format("{0}qe.aspx?pname={1}&name={2}&scope={3}",
                _baseAddress.AbsoluteUri, teamProjects, queryName, scope));
        }

        public Uri RunWorkItemQuery(string teamProjects, string queryName, string scope)
        {
            // Example: http://tfs:8090/q.aspx?pname=TeamProject&name=My%30Bugs&scope=private

            return new Uri(String.Format("{0}q.aspx?pname={1}&name={2}&scope={3}",
                _baseAddress.AbsoluteUri, teamProjects, queryName, scope));
        }

        public Uri ShowSourceFile(string versionControlPath)
        {
            // Example: http://tfs:8090/view.aspx?path=$/TeamProject/Main/dnp.cs

            return new Uri(String.Format("{0}view.aspx?path={1}",
                _baseAddress.AbsoluteUri, versionControlPath));
        }

        public Uri ShowSourceFile(int itemID)
        {
            // Example: http://tfs:8090/view.aspx?item=123

            return new Uri(String.Format("{0}view.aspx?item={1}",
                _baseAddress.AbsoluteUri, itemID));
        }

        public Uri ShowSourceHistory(string versionControlPath)
        {
            // Example: http://tfs:8090/history.aspx?path=$/TeamProject/Main/dnp.cs

            return new Uri(String.Format("{0}history.aspx?path={1}",
                _baseAddress.AbsoluteUri, versionControlPath));
        }

        public Uri ShowChangeset(int changesetID)
        {
            // Example: http://tfs:8090/cs.aspx?cs=1234

            return new Uri(String.Format("{0}cs.aspx?cs={1}",
                _baseAddress.AbsoluteUri, changesetID));
        }

        public Uri ShowShelveset(string shelvesetName, string owner)
        {
            // Example: http://tfs:8090/ss.aspx?ss=Shelvesetname;domain\UserName

            return new Uri(String.Format("{0}ss={1};{2}",
                _baseAddress.AbsoluteUri, shelvesetName, owner));
        }

        public Uri ShowSourceDiff(string versionControlPath1, string versionControlPath2)
        {
            // Example: http://tfs:8090/diff.aspx?opath=$/TeamProject/Main/dnp.cs&mpath=$/TeamProject/V1.0/dnp.cs

            return new Uri(String.Format("{0}diff.aspx?opath={1}&mpath={2}",
                _baseAddress.AbsoluteUri, versionControlPath1, versionControlPath2));
        }

        public Uri ShowSourceAnnotate(string versionControlPath)
        {
            // Example: http://tfs:8090/ann.aspx?path=$/TeamProject/Main/dnp.cs

            return new Uri(String.Format("{0}/ann.aspx?path={1}",
                _baseAddress.AbsoluteUri, versionControlPath));
        }

        public Uri ShowSourceAnnotate(string versionControlPath, int stopAtChangesetID)
        {
            // Example: http://tfs:8090/ann.aspx?path=$/TeamProject/Main/dnp.cs&cs=123

            return new Uri(String.Format("{0}/ann.aspx?path={1}&cs={2}",
                _baseAddress.AbsoluteUri, versionControlPath, stopAtChangesetID));
        }

        // Not implemented:
        // Show deleted file: http://tfs:8090/view.aspx?item=123&del=123
        // show diff: http://tfs:8090/diff.aspx?opath=$/TeamProject/Main/dnp.cs&ocs=100&mpath=$/TeamProject/V1.0/dnp.cs&mcs=200
    }
}
