using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Web;

namespace TeamSystemPro.Samples.WebAccessLinkCreator
{
    public partial class MainForm : Form
    {
        public MainForm()
        {
            InitializeComponent();

            this.Text = Application.ProductName;
            cboLinkType.SelectedIndex = 1;
        }

        private void txtInput_TextChanged(object sender, EventArgs e)
        {
            string input1 = txtInput1.Text;
            string input2 = txtInput2.Text;
            string input3 = txtInput3.Text;
            string serverUrl = txtServerUrl.Text;
            if (!serverUrl.StartsWith("http"))
            {
                serverUrl = "http://" + serverUrl;
            }
            if (!serverUrl.EndsWith("/"))
            {
                serverUrl += "/";
            }

            Properties.Settings.Default.Save();

            LinkTypes type = (LinkTypes)cboLinkType.SelectedIndex;

            if (String.IsNullOrEmpty(input1) ||
                (TypeRequiresIntInput(type) &&
                !IsNumeric(input1)))
            {
                txtResult.Clear();
                return;
            }

            LinkCreator creator = new LinkCreator(new Uri(serverUrl));

            Uri result = null;
            switch (type)
            {
                // Team Projects
                case LinkTypes.OpenTeamProject:
                    result = creator.OpenTeamProject(input1);
                    break;

                // Work Items
                case LinkTypes.OpenWorkItem:
                    result = creator.OpenWorkItem(Convert.ToInt32(input1));
                    break;
                case LinkTypes.CreateWorkItem:
                    result = creator.CreateWorkItem(input1, input2);
                    break;

                // Work Item Queries
                case LinkTypes.RunWorkItemQuery:
                    result = creator.RunWorkItemQuery(input1, input2, input3);
                    break;
                case LinkTypes.NewWorkItemQuery:
                    result = creator.NewWorkItemQuery(input1);
                    break;
                case LinkTypes.EditWorkItemQuery:
                    result = creator.EditWorkItemQuery(input1, input2, input3);
                    break;

                // Source Control
                case LinkTypes.ShowSourceFileByFilename:
                    result = creator.ShowSourceFile(input1);
                    break;
                case LinkTypes.ShowSourceFileByItemID:
                    result = creator.ShowSourceFile(Convert.ToInt32(input1));
                    break;
                case LinkTypes.ShowChangeset:
                    result = creator.ShowChangeset(Convert.ToInt32(input1));
                    break;
                case LinkTypes.ShowShelveset:
                    result = creator.ShowShelveset(input1, input2);
                    break;
                case LinkTypes.ShowSourceHistory:
                    result = creator.ShowSourceHistory(input1);
                    break;
                case LinkTypes.ShowSourceDiff:
                    result = creator.ShowSourceDiff(input1, input2);
                    break;
                case LinkTypes.ShowSourceAnnotate:
                    if (String.IsNullOrEmpty(input2) || !IsNumeric(input2))
                    {
                        result = creator.ShowSourceAnnotate(input1);
                    }
                    else
                    {
                        result = creator.ShowSourceAnnotate(input1, Convert.ToInt32(input2));
                    }
                    break;
            }
            txtResult.Text = result.AbsoluteUri;

            CopyToClipboard();
        }

        private bool TypeRequiresIntInput(LinkTypes type)
        {
            return (type == LinkTypes.OpenWorkItem ||
                type == LinkTypes.ShowSourceFileByItemID ||
                type == LinkTypes.ShowChangeset);
        }

        private bool IsNumeric(string input)
        {
            foreach (char c in input.ToCharArray())
            {
                if (!Char.IsNumber(c))
                {
                    return false;
                }
            }
            return true;
        }

        private void CopyToClipboard()
        {
            Clipboard.SetText(txtResult.Text, TextDataFormat.Text);
        }

        private void cboLinkType_SelectedIndexChanged(object sender, EventArgs e)
        {
            LinkTypes type = (LinkTypes)cboLinkType.SelectedIndex;

            txtInput1.Clear();
            txtInput2.Clear();
            txtInput3.Clear();

            switch (type)
            {
                case LinkTypes.OpenTeamProject:
                case LinkTypes.NewWorkItemQuery:
                    NeedsInput(1);
                    lblInput1.Text = "Team Project Name:";
                    break;
                case LinkTypes.CreateWorkItem:
                    NeedsInput(2);
                    lblInput1.Text = "Team Project Name:";
                    lblInput2.Text = "Work Item Type:  (e.g. \"Bug\")";
                    break;
                case LinkTypes.EditWorkItemQuery:
                case LinkTypes.RunWorkItemQuery:
                    NeedsInput(3);    
                    lblInput1.Text = "Team Project Name:";
                    lblInput2.Text = "Work Item Query name:  (e.g. \"Bug\")";
                    lblInput3.Text = "Query Scope (\"public\" or \"private\"):";
                    txtInput3.Text = "public";
                    break;
                case LinkTypes.OpenWorkItem:
                    NeedsInput(1);
                    lblInput1.Text = "Work Item ID:";
                    break;
                case LinkTypes.ShowSourceFileByItemID:
                    NeedsInput(1);
                    lblInput1.Text = "Source File ID:";
                    break;
                case LinkTypes.ShowSourceFileByFilename:
                case LinkTypes.ShowSourceHistory:
                    NeedsInput(1);
                    lblInput1.Text = "Source File Name:  (e.g. $/Folder/file.cs)";
                    break;
                case LinkTypes.ShowSourceAnnotate:
                    NeedsInput(2);
                    lblInput1.Text = "Source File Name:  (e.g. $/Folder/file.cs)";
                    lblInput2.Text = "Stop at Changeset # (optional):";
                    break;
                case LinkTypes.ShowChangeset:
                    NeedsInput(1);
                    lblInput1.Text = "Changeset #:";
                    break;
                case LinkTypes.ShowShelveset:
                    NeedsInput(2);
                    lblInput1.Text = "Shelveset name:";
                    lblInput2.Text = @"Owner  (e.g. DOMAIN\Account):";
                    break;
                case LinkTypes.ShowSourceDiff:
                    NeedsInput(2);
                    lblInput1.Text = "Original file:  (e.g. $/Folder/file1.cs)";
                    lblInput2.Text = "File to compare:  (e.g. $/Folder/file2.cs)";
                    break;
            }

            txtInput1.Focus();
        }

        private void NeedsInput(int number)
        {
            if (number < 1 || number > 3)
                throw new ArgumentOutOfRangeException("number");

            switch (number)
            {
                case 1:
                    txtInput2.Visible = lblInput2.Visible = false;
                    txtInput3.Visible = lblInput3.Visible = false;
                    break;
                case 2:
                    txtInput2.Visible = lblInput2.Visible = true;
                    break;
                case 3:
                    txtInput2.Visible = lblInput2.Visible = true;
                    txtInput3.Visible = lblInput3.Visible = true;
                    break;
            }
        }

        private void cmdCopy_Click(object sender, EventArgs e)
        {
            CopyToClipboard();
        }

        private void txtResult_TextChanged(object sender, EventArgs e)
        {
            cmdCopy.Enabled = (txtResult.Text.Length > 0);
        }
    }

    public enum LinkTypes
    {
        // Team Projects
        OpenTeamProject = 0,

        // Work Items
        OpenWorkItem = 1,
        CreateWorkItem = 2,

        // Work Item Queries
        RunWorkItemQuery = 3,
        NewWorkItemQuery = 4,
        EditWorkItemQuery = 5,

        // Source Control
        ShowSourceFileByFilename = 6,
        ShowSourceFileByItemID = 7,
        ShowSourceHistory = 8,
        ShowChangeset = 9,
        ShowShelveset = 10,
        ShowSourceDiff = 11,
        ShowSourceAnnotate = 12
    }
}